searchNodes=[{"type":"module","title":"K8sWebhoox","doc":"Kubernetes webhook endpoints need to be TLS terminated. This module and the only exported functions ensure_certificates/5 , update_admission_webhook_configs/3 and update_crd_conversion_configs/3 help initializing TLS termination for your webhook endpoint. How it works The function ensure_certificates/5 generates a CA and a SSL certificate and stores them in a secret, together with their private keys. If the secret already exists, it reads the certificates from that secret and doesn't generate them again. update_admission_webhook_configs/3 searches the cluster for resources of type admissionregistration.k8s.io/v1/ValidatingWebhookConfiguration and admissionregistration.k8s.io/v1/MutatingWebhookConfiguration and updates them in place, setting the caBundle entry to the value of the generated CA certificate. update_crd_conversion_configs/3 searches the cluster for resources of type apiextensions.k8s.io/v1/CustomResourceDefinition and updates them in place, setting the caBundle entry to the value of the generated CA certificate. Usage RBAC I assume you already have a Kubernetes deployment running your applicaiton bundled with this library. Make sure the service account used by the pods can read and patch secrets and webhook configurations. You can create and assign it the following RBAC role: apiVersion: rbac.authorization.k8s.io/v1 kind: Role metadata: name: your-operator namespace: default rules: - apiGroups: - admissionregistration.k8s.io resources: - validatingwebhookconfigurations - mutatingwebhookconfigurations verbs: [&quot;*&quot;] - apiGroups: [&quot;&quot;] resources: [&quot;secrets&quot;, &quot;customresourcedefinitions&quot;] verbs: [&quot;*&quot;] Mounting the certificates In the main container of your deployment, mount a secret as volume. In the example below, the secret is called admission-webhook-cert and is mounted to /mnt/cert . The path should correlate to where you load the certificate from in your HTTP Server configuration (see K8sWebhoox.Plug ). The secret does not exist at this moment which is why we set optional: true : apiVersion: apps/v1 kind: Deployment ... spec: template: spec: containers: - name: your-operator ... volumeMounts: - name: cert mountPath: &quot;/mnt/cert&quot; readOnly: true volumes: - name: cert secret: secretName: admission-webhook-cert optional: true Use Init Container to Create Certificates In your code base, implement a function that calls the functions of this module in order to initialize the TLS configuration: defmodule MyOperator.K8sWebhooxTLS do def bootstrap_tls ( ) do # Initialize connection to Kubernetes conn = ... # Create certificates if necessary { :ok , ca_bundle_base_64 } = K8sWebhoox . ensure_certificates ( conn , &quot;default&quot; , &quot;my-operator&quot; , &quot;default&quot; , &quot;webhook-tls-certificate&quot; ) # Patch admission webhook configurations :ok = K8sWebhoox . update_admission_webhook_configs ( conn , &quot;my-operator&quot; , ca_bundle_base_64 ) # Patch conversion webhook configurations in CRDs :ok = K8sWebhoox . update_crd_conversion_configs ( conn , &quot;myoperator.com&quot; , ca_bundle_base_64 ) end end In your deployment, add an init container with the same image as the main container and call your TLS bootstrap function using eval : ... initContainers: - name: bootstrap-tls image: SAME_AS_MAIN_CONTAINER args: [&quot;eval&quot;, &quot;MyOperator.K8sWebhooxTLS.bootstrap_tls()&quot;] ...","ref":"K8sWebhoox.html"},{"type":"function","title":"K8sWebhoox.ensure_certificates/6","doc":"Gets the certificate bundle from the Kubernetes Secret. Creates new CA and certificate if necessary. Returns the CA bundle Base64 encoded.","ref":"K8sWebhoox.html#ensure_certificates/6"},{"type":"function","title":"K8sWebhoox.update_admission_webhook_configs/3","doc":"Searches the cluster for ValidatingWebhookConfiguration and MutatingWebhookConfiguration resources with the given admission_config_name and sets the .webhooks[*].clientConfig.caBundle fields to ca_bundle_base_64","ref":"K8sWebhoox.html#update_admission_webhook_configs/3"},{"type":"function","title":"K8sWebhoox.update_crd_conversion_configs/3","doc":"Searches the cluster for CustomResourceDefinition resources for the given group and sets the .spec.conversion.webhook.clientConfig.caBundle fields to ca_bundle_base_64 .","ref":"K8sWebhoox.html#update_crd_conversion_configs/3"},{"type":"module","title":"K8sWebhoox.AdmissionControl.AdmissionReview","doc":"This module defines a struct which is used as token in the Pluggable pipeline handling an admission request. See K8sWebhoox.Plug for more information on how to set up the request handler pipeline. This module also defines a set of useful helpers when processing an admission request.","ref":"K8sWebhoox.AdmissionControl.AdmissionReview.html"},{"type":"function","title":"K8sWebhoox.AdmissionControl.AdmissionReview.add_warning/2","doc":"Adds a warning to the admission review's response. Examples iex&gt; conn = % K8sWebhoox.Conn { request : %{ } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . add_warning ( conn , &quot;warning&quot; ) % K8sWebhoox.Conn { request : %{ } , response : %{ &quot;warnings&quot; =&gt; [ &quot;warning&quot; ] } , api_version : &quot;&quot; , kind : &quot;&quot; } iex&gt; conn = % K8sWebhoox.Conn { request : %{ } , response : %{ &quot;warnings&quot; =&gt; [ &quot;existing_warning&quot; ] } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . add_warning ( conn , &quot;new_warning&quot; ) % K8sWebhoox.Conn { request : %{ } , response : %{ &quot;warnings&quot; =&gt; [ &quot;new_warning&quot; , &quot;existing_warning&quot; ] } , api_version : &quot;&quot; , kind : &quot;&quot; }","ref":"K8sWebhoox.AdmissionControl.AdmissionReview.html#add_warning/2"},{"type":"function","title":"K8sWebhoox.AdmissionControl.AdmissionReview.allow/1","doc":"Responds by allowing the operation Examples iex&gt; conn = % K8sWebhoox.Conn { request : %{ } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . allow ( conn ) % K8sWebhoox.Conn { request : %{ } , response : %{ &quot;allowed&quot; =&gt; true } , api_version : &quot;&quot; , kind : &quot;&quot; }","ref":"K8sWebhoox.AdmissionControl.AdmissionReview.html#allow/1"},{"type":"function","title":"K8sWebhoox.AdmissionControl.AdmissionReview.check_allowed_values/4","doc":"Checks the given field's value - if defined - against a list of allowed values. If the field is not defined, the request is considered valid and no error is returned. Use the CRD to define required fields. Examples iex&gt; conn = % K8sWebhoox.Conn { request : %{ &quot;object&quot; =&gt; %{ &quot;metadata&quot; =&gt; %{ &quot;annotations&quot; =&gt; %{ &quot;some/annotation&quot; =&gt; &quot;bar&quot; } } , &quot;spec&quot; =&gt; %{ } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ } } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . check_allowed_values ( conn , ~w(metadata annotations some/annotation) , [ &quot;foo&quot; , &quot;bar&quot; ] , &quot;.metadata.annotations.some/annotation&quot; ) % K8sWebhoox.Conn { request : %{ &quot;object&quot; =&gt; %{ &quot;metadata&quot; =&gt; %{ &quot;annotations&quot; =&gt; %{ &quot;some/annotation&quot; =&gt; &quot;bar&quot; } } , &quot;spec&quot; =&gt; %{ } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ } } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } iex&gt; conn = % K8sWebhoox.Conn { request : %{ &quot;object&quot; =&gt; %{ &quot;metadata&quot; =&gt; %{ } , &quot;spec&quot; =&gt; %{ } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ } } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . check_allowed_values ( conn , ~w(metadata annotations some/annotation) , [ &quot;foo&quot; , &quot;bar&quot; ] , &quot;.metadata.annotations.some/annotation&quot; ) % K8sWebhoox.Conn { request : %{ &quot;object&quot; =&gt; %{ &quot;metadata&quot; =&gt; %{ } , &quot;spec&quot; =&gt; %{ } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ } } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } iex&gt; conn = % K8sWebhoox.Conn { request : %{ &quot;object&quot; =&gt; %{ &quot;metadata&quot; =&gt; %{ &quot;annotations&quot; =&gt; %{ &quot;some/annotation&quot; =&gt; &quot;other&quot; } } , &quot;spec&quot; =&gt; %{ } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ } } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . check_allowed_values ( conn , ~w(metadata annotations some/annotation) , [ &quot;foo&quot; , &quot;bar&quot; ] , &quot;.metadata.annotations.some/annotation&quot; ) % K8sWebhoox.Conn { request : %{ &quot;object&quot; =&gt; %{ &quot;metadata&quot; =&gt; %{ &quot;annotations&quot; =&gt; %{ &quot;some/annotation&quot; =&gt; &quot;other&quot; } } , &quot;spec&quot; =&gt; %{ } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ } } } , response : %{ &quot;allowed&quot; =&gt; false , &quot;status&quot; =&gt; %{ &quot;code&quot; =&gt; 400 , &quot;message&quot; =&gt; ~S(The field .metadata.annotations.some/annotation must contain one of the values in [&quot;foo&quot;, &quot;bar&quot;] but it&#39;s currently set to &quot;other&quot;.) } } , api_version : &quot;&quot; , kind : &quot;&quot; }","ref":"K8sWebhoox.AdmissionControl.AdmissionReview.html#check_allowed_values/4"},{"type":"function","title":"K8sWebhoox.AdmissionControl.AdmissionReview.check_immutable/2","doc":"Defines a field as being immutable. Denies the request if the field was mutated. Examples iex&gt; conn = % K8sWebhoox.Conn { request : %{ &quot;operation&quot; =&gt; &quot;UPDATE&quot; , &quot;object&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;value&quot; } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;value&quot; } } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . check_immutable ( conn , [ &quot;spec&quot; , &quot;immutable&quot; ] ) % K8sWebhoox.Conn { request : %{ &quot;operation&quot; =&gt; &quot;UPDATE&quot; , &quot;object&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;value&quot; } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;value&quot; } } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } iex&gt; conn = % K8sWebhoox.Conn { request : %{ &quot;operation&quot; =&gt; &quot;UPDATE&quot; , &quot;object&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;new_value&quot; } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;value&quot; } } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . check_immutable ( conn , [ &quot;spec&quot; , &quot;immutable&quot; ] ) % K8sWebhoox.Conn { request : %{ &quot;operation&quot; =&gt; &quot;UPDATE&quot; , &quot;object&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;new_value&quot; } } , &quot;oldObject&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;value&quot; } } } , response : %{ &quot;allowed&quot; =&gt; false , &quot;status&quot; =&gt; %{ &quot;code&quot; =&gt; 400 , &quot;message&quot; =&gt; &quot;The field .spec.immutable is immutable.&quot; } } , api_version : &quot;&quot; , kind : &quot;&quot; } iex&gt; conn = % K8sWebhoox.Conn { request : %{ &quot;operation&quot; =&gt; &quot;CREATE&quot; , &quot;object&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;new_value&quot; } } , &quot;oldObject&quot; =&gt; %{ } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . check_immutable ( conn , [ &quot;spec&quot; , &quot;immutable&quot; ] ) % K8sWebhoox.Conn { request : %{ &quot;operation&quot; =&gt; &quot;CREATE&quot; , &quot;object&quot; =&gt; %{ &quot;spec&quot; =&gt; %{ &quot;immutable&quot; =&gt; &quot;new_value&quot; } } , &quot;oldObject&quot; =&gt; %{ } } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; }","ref":"K8sWebhoox.AdmissionControl.AdmissionReview.html#check_immutable/2"},{"type":"function","title":"K8sWebhoox.AdmissionControl.AdmissionReview.deny/1","doc":"Responds by denying the operation Examples iex&gt; conn = % K8sWebhoox.Conn { request : %{ } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . deny ( conn ) % K8sWebhoox.Conn { request : %{ } , response : %{ &quot;allowed&quot; =&gt; false } , api_version : &quot;&quot; , kind : &quot;&quot; }","ref":"K8sWebhoox.AdmissionControl.AdmissionReview.html#deny/1"},{"type":"function","title":"K8sWebhoox.AdmissionControl.AdmissionReview.deny/3","doc":"Responds by denying the operation, returning response code and message Examples iex&gt; conn = % K8sWebhoox.Conn { request : %{ } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } ...&gt; K8sWebhoox.AdmissionControl.AdmissionReview . deny ( conn , 403 , &quot;foo&quot; ) % K8sWebhoox.Conn { request : %{ } , response : %{ &quot;allowed&quot; =&gt; false , &quot;status&quot; =&gt; %{ &quot;code&quot; =&gt; 403 , &quot;message&quot; =&gt; &quot;foo&quot; } } , api_version : &quot;&quot; , kind : &quot;&quot; } iex&gt; K8sWebhoox.AdmissionControl.AdmissionReview . deny ( % K8sWebhoox.Conn { request : %{ } , response : %{ } , api_version : &quot;&quot; , kind : &quot;&quot; } , &quot;foo&quot; ) % K8sWebhoox.Conn { request : %{ } , response : %{ &quot;allowed&quot; =&gt; false , &quot;status&quot; =&gt; %{ &quot;code&quot; =&gt; 400 , &quot;message&quot; =&gt; &quot;foo&quot; } } , api_version : &quot;&quot; , kind : &quot;&quot; }","ref":"K8sWebhoox.AdmissionControl.AdmissionReview.html#deny/3"},{"type":"module","title":"K8sWebhoox.AdmissionControl.Handler","doc":"A Helper module for admission review request handling. When use d, it turns the using module into a Pluggable step which can be used with K8sWebhoox.Plug . The :webhook_type option has to be set to either :validating or :mutating when initializing the Pluggable : post &quot;/k8s-webhooks/admission-review/validating&quot; , to : K8sWebhoox.Plug , init_opts : [ webhook_handler : { MyOperator.AdmissionControlHandler , webhook_type : :validating } ] Usage defmodule MyOperator.AdmissionControlHandler do use K8sWebhoox.AdmissionControl.Handler alias K8sWebhoox.AdmissionControl.AdmissionReview # Mutate someresources resource mutate &quot;example.com/v1/someresources&quot; , conn do AdmissionReview . deny ( conn ) end # Validate the sacle subresource of a pod validate &quot;v1/pods&quot; , &quot;scale&quot; , conn do conn end end","ref":"K8sWebhoox.AdmissionControl.Handler.html"},{"type":"macro","title":"K8sWebhoox.AdmissionControl.Handler.mutate/4","doc":"Defines a handler for mutating webhook requests. The resource this handler mutates is defined in the form &quot;group/version/plural&quot; (plural being the plural form of the resource, e.g. &quot;deployments&quot;). The subresource is optional. If given, the handler is only called for mutation of the given subresource . The parameter conn_var defines the variable name of the %K8sWebhoox.Conn{} token inside your handler. Example mutate &quot;example.com/v1/myresources&quot; , conn do # your mutations conn end Validating the scale subresource: mutate &quot;v1/pod&quot; , &quot;scale&quot; , conn do # your mutations conn end","ref":"K8sWebhoox.AdmissionControl.Handler.html#mutate/4"},{"type":"macro","title":"K8sWebhoox.AdmissionControl.Handler.validate/4","doc":"Defines a handler for validating webhook requests. The resource this handler validates is defined in the form &quot;group/version/plural&quot; (plural being the plural form of the resource, e.g. &quot;deployments&quot;). The subresource is optional. If given, the handler is only called for validation of the given subresource . The parameter conn_var defines the variable name of the %K8sWebhoox.Conn{} token inside your handler. Example validate &quot;example.com/v1/myresources&quot; , conn do # your validations conn end Validating the &quot;scale&quot; subresource: validate &quot;example.com/v1/myresources&quot; , &quot;scale&quot; , conn do # your validations conn end You can use the K8sWebhoox.AdmissionControl.AdmissionReview helper module to validate the request: validate &quot;v1/pod&quot; , &quot;scale&quot; , conn do # the &quot;some_label&quot; is immutable K8sWebhoox.AdmissionControl.AdmissionReview . check_immutable ( conn , [ &quot;metadata&quot; , &quot;labels&quot; , &quot;some_lable&quot; ] ) end","ref":"K8sWebhoox.AdmissionControl.Handler.html#validate/4"},{"type":"module","title":"K8sWebhoox.Conn","doc":"This module defines a struct which is used as token in the Pluggable pipeline handling of webhook requests. There are several modules defining helpers that operate on these tokens: K8sWebhoox.AdmissionControl.AdmissionReview - Helpers when handling admission control webhook requests","ref":"K8sWebhoox.Conn.html"},{"type":"function","title":"K8sWebhoox.Conn.new/2","doc":"","ref":"K8sWebhoox.Conn.html#new/2"},{"type":"type","title":"K8sWebhoox.Conn.t/0","doc":"The struct used as token in the request handler pipeline. Fields api_version - The API Version of the received resource kind - The Kind of the received resource request - The body of the HTTPS request representing the admission request. response - The resposne the request handler pipeline is suposed to define. Internal Fields halted - Whether the pipeline is halted or not. Defaults to false . assigns - A map used to internally forward data within the pipeline. Defaults to %{} .","ref":"K8sWebhoox.Conn.html#t:t/0"},{"type":"module","title":"K8sWebhoox.Plug","doc":"A Plug used to handle admission webhook requests. The Plug, when called, extracts the admission request from %Plug.Conn{} and passes a %K8sWebhoox.Conn{} to the handlers in the pipeline. Usage Plug it in Once your endpoint serves via HTTPS, you can route admission webhook requests to this Plug as follows: defmodule MyOperator.Router do use Plug.Router plug :match plug :dispatch post &quot;/admission-review/validating&quot; , to : K8sWebhoox.Plug , init_opts : [ webhook_handler : { MyOperator.K8sWebhoox.AdmissionControlHandler , webhook_type : :validating } ] end Implementing the Handler The webhook handler ( MyOperator.K8sWebhoox.AdmissionControlHandler in the example above) needs to implement the Pluggable behviour. Pluggable is very simliar to Plug but instead of a %Plug.Conn{} , you get a %K8sWebhoox.Conn{} struct passed to call/2 . Use the helper functions in K8sWebhoox.AdmissionControl.AdmissionReview to process the request. defmodule MyOperator.K8sWebhoox.AdmissionControlHandler do @behaviour Pluggable alias K8sWebhoox.AdmissionControl.AdmissionReview def init ( _ ) , do : nil def call ( %{ assigns : %{ webhook_type : :validation } } = conn , _ ) do case conn . request [ &quot;resource&quot; ] do %{ &quot;group&quot; =&gt; &quot;my-operator.com&quot; , &quot;version&quot; =&gt; &quot;v1beta1&quot; , &quot;resource&quot; =&gt; &quot;mycrd&quot; } -&gt; AdmissionReview . check_immutable ( conn , [ &quot;spec&quot; , &quot;some_immutable_field&quot; ] ) _ -&gt; conn end end Options The plug has to be initialized with to mandatory option webhook_handler : webhook_handler - The Pluggable handling the admission request. Can be a module or a tuple in the form {Handler.Module, init_opts} . The latter will pass the init_opts to the init/1 function of the handler: post &quot;/k8s-webhook&quot; , to : K8sWebhoox.Plug , init_opts : [ webhook_handler : { MyOperator.K8sWebhoox.RequestHandler , env : env } ]","ref":"K8sWebhoox.Plug.html"},{"type":"behaviour","title":"K8sWebhoox.ResourceConversion.Handler","doc":"A Helper module for resource conversion request handling. When use d, it turns the using module into a Pluggable step which can be used with K8sWebhoox.Plug . post &quot;/k8s-webhooks/resource-conversion&quot; , to : K8sWebhoox.Plug , init_opts : [ webhook_handler : MyOperator.ResourceConversionHandler ] Usage Declare convert to convert a given resource to a desired_api_version . defmodule MyOperator.ResourceConversionHandler do use K8sWebhoox.AdmissionControl.Handler def convert ( %{ &quot;apiVersion&quot; =&gt; &quot;example.com/v1beta1&quot; , &quot;kind&quot; =&gt; &quot;MyResource&quot; } = resource , &quot;example.com/v1&quot; ) do # return {:ok, mutated_resource} { :ok , put_in ( resource , ~w(metadata labels) , %{ &quot;foo&quot; =&gt; &quot;bar&quot; } ) } end def convert ( %{ &quot;apiVersion&quot; =&gt; &quot;example.com/v1alpha1&quot; , &quot;kind&quot; =&gt; &quot;MyResource&quot; } = resource , &quot;example.com/v1&quot; ) do # return {:error, message} { :error , &quot;V1Alpha1 cannot be converted to V1.&quot; } end end","ref":"K8sWebhoox.ResourceConversion.Handler.html"},{"type":"callback","title":"K8sWebhoox.ResourceConversion.Handler.convert/2","doc":"Defines a handler for a converstion request of the given resource to the given desired_api_version . See moduledoc for an example.","ref":"K8sWebhoox.ResourceConversion.Handler.html#c:convert/2"},{"type":"extras","title":"K8sWebhoox - Kubernetes Webhooks SDK for Elixir","doc":"","ref":"readme.html"},{"type":"extras","title":"K8sWebhoox - Kubernetes Webhooks SDK for Elixir - Installation","doc":"def deps do [ { :k8s_webhoox , &quot;~&gt; 0.2.0&quot; } ] end","ref":"readme.html#installation"},{"type":"extras","title":"K8sWebhoox - Kubernetes Webhooks SDK for Elixir - Prerequisites","doc":"In order to process Kubernetes webhook requests, your endpoint needs TLS termination. You can use the K8sWebhoox helper module to bootstrap TLS using an initContainer . Once the certificates are generated and mounted (e.g. to /mnt/cert/tls.crt and /mnt/cert/tls.key ), you can initialize Bandit or Cowboy in your application.ex to serve your webhook requests via HTTPS: defmodule MyOperator.Application do def start ( _type , env : env ) do children = [ { Bandit , plug : MyOperator.Router , port : 4000 , certfile : &quot;/mnt/cert/tls.crt&quot; , keyfile : &quot;/mnt/cert/tls.key&quot; , scheme : :https } ] opts = [ strategy : :one_for_one , name : MyOperator.Supervisor ] Supervisor . start_link ( children , opts ) end end","ref":"readme.html#prerequisites"},{"type":"extras","title":"K8sWebhoox - Kubernetes Webhooks SDK for Elixir - Router Implementation","doc":"In your router Plug, you can forward webhook requests to K8sWebhoox.Plug as follows: defmodule MyOperator.Router do use Plug.Router plug :match plug :dispatch post &quot;/admission-review/mutating&quot; , to : K8sWebhoox.Plug , init_opts : [ webhook_handler : { MyOperator.K8sWebhoox.AdmissionControlHandler , webhook_type : :mutating } ] post &quot;/admission-review/validating&quot; , to : K8sWebhoox.Plug , init_opts : [ webhook_handler : { MyOperator.K8sWebhoox.AdmissionControlHandler , webhook_type : :validating } ] post &quot;/resource-conversion&quot; , to : K8sWebhoox.Plug , init_opts : [ webhook_handler : MyOperator.K8sWebhoox.ResourceConversionHandler ] end","ref":"readme.html#router-implementation"},{"type":"extras","title":"K8sWebhoox - Kubernetes Webhooks SDK for Elixir - Handler Implementation","doc":"Webhook request handlers are Pluggable steps with a %K8sWebhoox.Conn{} struct passed as token. You can ipmlement them from scratch or use the helper modules as described below. Admission Control Handlers You may use K8sWebhoox.AdmissionControl.Handler in your module to simplify the implementation of an admission webhook request handler. defmodule MyOperator.AdmissionControlHandler do use K8sWebhoox.AdmissionControl.Handler alias K8sWebhoox.AdmissionControl.AdmissionReview # Mutate someresources resource mutate &quot;example.com/v1/someresources&quot; , conn do AdmissionReview . deny ( conn ) end # Validate the sacle subresource of a pod validate &quot;v1/pods&quot; , &quot;scale&quot; , conn do # Use the helper functions defined in `K8sWebhoox.AdmissionControl.AdmissionReview`. conn end end Resource Conversion Handlers You may use K8sWebhoox.ResourceConversion.Handler in your module to simplify the implementation of a resource conversion webhook request handler. defmodule MyOperator.ResourceConversionHandler do use K8sWebhoox.AdmissionControl.Handler def convert ( %{ &quot;apiVersion&quot; =&gt; &quot;example.com/v1beta1&quot; , &quot;kind&quot; =&gt; &quot;MyResource&quot; } = resource , &quot;example.com/v1&quot; ) do # return {:ok, mutated_resource} { :ok , put_in ( resource , ~w(metadata labels) , %{ &quot;foo&quot; =&gt; &quot;bar&quot; } ) } end def convert ( %{ &quot;apiVersion&quot; =&gt; &quot;example.com/v1alpha1&quot; , &quot;kind&quot; =&gt; &quot;MyResource&quot; } = resource , &quot;example.com/v1&quot; ) do # return {:error, message} { :error , &quot;V1Alpha1 cannot be converted to V1.&quot; } end end","ref":"readme.html#handler-implementation"},{"type":"extras","title":"Changelog","doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html"},{"type":"extras","title":"Changelog - Unreleased","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"Changelog - [0.2.0] - 2023-07-12","doc":"Changed TLS secret creation: use keys that correlate with keys in secrets created by cert-manager #17","ref":"changelog.html#0-2-0-2023-07-12"},{"type":"extras","title":"Changelog - [0.1.1] - 2023-06-02","doc":"Fixed K8sWebhoox.AdmissionControl.AdmissionReview.check_immutable/2 - only check UPDATE operations. Changed K8sWebhoox.AdmissionControl.AdmissionReview.check_allowed_values/4 - Add field_name used for error message. Added TLS Bootstrap - Renews certificates if they expire within 30 days from now - #5","ref":"changelog.html#0-1-1-2023-06-02"},{"type":"extras","title":"Changelog - [0.1.0] - 2023-04-09","doc":"This is the first release.","ref":"changelog.html#0-1-0-2023-04-09"}]